﻿using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using System;
using System.Linq;

namespace EFCore5Console
{
  internal class Program
  {
   
    private static void Main(string[] args)
    {

      using (var context = new PeopleContext())
      {
        context.Database.EnsureDeleted();
        context.Database.EnsureCreated();
      }


        Console.Clear();
      //QueryM2M();
      // FilteredInclude();
     // SplitQueryInclude();
      TPTWork();
    }

    private static void TPTWork()
    {
      using (var context = new PeopleContext())
      {
        context.AddRange(
          new ScaryWildlifeSighting { AddressId = 1, DateTime = DateTime.Now, Description = "Bear", Experience = "OMG I almost peed my pants" },
          new ScaryWildlifeSighting { AddressId = 1, DateTime = DateTime.Now, Description = "Snake", Experience = "I think I scared that poor snake" });
        context.SaveChanges();
        context.ChangeTracker.Clear();
        var sighting = context.WildlifeSightings.OfType<ScaryWildlifeSighting>().Where(s => s.Experience.Contains("peed")).ToList();
      }
    }

    private static void SplitQueryInclude()
    {
      using (var context = new PeopleContext())
      {
        Console.WriteLine("Split Query");
        var address = context.Addresses.Include(a => a.WildlifeSightings).FirstOrDefault();
        var address2 = context.Addresses.AsSplitQuery().Include(a => a.WildlifeSightings).FirstOrDefault();

      }
    }

    private static void FilteredInclude()
    {
      using (var context = new PeopleContext())
      {
        Console.WriteLine("No Filter/Sort");
        var address = context.Addresses.Include(a => a.WildlifeSightings).FirstOrDefault();
        OutPutWildlife(address);
        context.ChangeTracker.Clear();
        Console.WriteLine("Filter");
        var a_bears = context.Addresses.Include(a => a.WildlifeSightings.Where(w => w.Description.Contains("Bear"))).FirstOrDefault();
        OutPutWildlife(a_bears);
        context.ChangeTracker.Clear();
        Console.WriteLine("Filter and Sort");
        var a_nobears = context.Addresses.Include(a => a.WildlifeSightings.OrderBy(w => w.Description).Where(w => !w.Description.Contains("Bear"))).FirstOrDefault();
        OutPutWildlife(a_nobears);
      }
    }

    private static void OutPutWildlife(Address input)
    {
      Console.WriteLine(input.Street);
      foreach (var wildlife in input.WildlifeSightings)
      {
        Console.WriteLine($"   {wildlife.DateTime}: {wildlife.Description}");
      }
    }

    private static void QueryM2M()
    {
      var person = new Person { FirstName = "Aaaaa", LastName = "Bbbbb" };
      var add999 = new Address { Street = "999 Main" };
      var add1000 = new Address { Street = "1000 Main" };
      person.Addresses.AddRange(new Address[] { add999, add1000 });

      using (var context = new PeopleContext())
      {
        var adev = context.People.FirstOrDefault();
        adev.Addresses.Add(person.Addresses[0]);
        context.People.Add(person);
        context.SaveChanges();
      }
      using (var context = new PeopleContext())
      {
        foreach (var p in context.People.Include(p => p.Addresses))
        {
          Console.WriteLine(p.FirstName);
          foreach (var a in p.Addresses)
          {
            Console.WriteLine($"**{a.Street}");
          }
        }
      }
    }
  }
}